@@ -30,6 +30,8 @@
 #include "sys/exception.hpp"
 #include <algorithm>
 #include <climits>
+#include <iostream>
+#include <iomanip>
 
 #define RESERVED_REG_NUM_FOR_SPILL 6
 
@@ -41,6 +43,55 @@ namespace gbe
 
   /*! Provides the location of a register in a vector */
   typedef std::pair<SelectionVector*, uint32_t> VectorLocation;
+  /*! Interval as used in linear scan allocator. Basically, stores the first and
+   *  the last instruction where the register is alive
+   */
+  struct GenRegInterval {
+    INLINE GenRegInterval(ir::Register reg) :
+      reg(reg), minID(INT_MAX), maxID(-INT_MAX) {}
+    ir::Register reg;     //!< (virtual) register of the interval
+    int32_t minID, maxID; //!< Starting and ending points
+  };
+
+  typedef struct GenRegIntervalKey {
+    GenRegIntervalKey(uint16_t reg, uint16_t maxID) {
+      if (maxID == INT_MAX)
+        maxID = 0xFFFF;
+      GBE_ASSERT(reg <= 0xFFFF && maxID <= 0xFFFF);
+      key = (maxID << 16) | reg;
+    }
+    const ir::Register getReg() const {
+      return (ir::Register)(key & 0xFFFF);
+    }
+    const uint16_t getMaxID() const {
+      return key >> 16;
+    }
+    uint32_t key;
+  } GenRegIntervalKey;
+
+  struct spillCmp {
+    bool operator () (const GenRegIntervalKey &lhs, const GenRegIntervalKey &rhs) const
+    { return lhs.key > rhs.key; }
+  };
+
+  typedef set <GenRegIntervalKey, spillCmp> SpillSet;
+
+  class SpillCandidateSet : public SpillSet
+  {
+  public:
+    std::set<GenRegIntervalKey, spillCmp>::iterator find(GenRegInterval interval) {
+      GenRegIntervalKey key(interval.reg, interval.maxID);
+      return SpillSet::find(key);
+    }
+    void insert(GenRegInterval interval) {
+      GenRegIntervalKey key(interval.reg, interval.maxID);
+      SpillSet::insert(key);
+    }
+    void erase(GenRegInterval interval) {
+      GenRegIntervalKey key(interval.reg, interval.maxID);
+      SpillSet::erase(key);
+    }
+  };
 
   /*! Implements the register allocation */
   class GenRegAllocator::Opaque
@@ -56,6 +107,20 @@ namespace gbe
     GenRegister genReg(const GenRegister &reg);
     /*! Output the register allocation */
     void outputAllocation(void);
+    INLINE void getRegAttrib(ir::Register reg, uint32_t &regSize, ir::RegisterFamily *regFamily = NULL) const {
+      // Note that byte vector registers use two bytes per byte (and can be
+      // interleaved)
+      static const size_t familyVectorSize[] = {2,2,2,4,8};
+      static const size_t familyScalarSize[] = {2,1,2,4,8};
+      using namespace ir;
+      const bool isScalar = ctx.sel->isScalarOrBool(reg);
+      const RegisterData regData = ctx.sel->getRegisterData(reg);
+      const RegisterFamily family = regData.family;
+      const uint32_t typeSize = isScalar ? familyScalarSize[family] : familyVectorSize[family];
+      regSize = isScalar ? typeSize : ctx.getSimdWidth() * typeSize;
+      if (regFamily != NULL)
+        *regFamily = family;
+    }
   private:
     /*! Expire one GRF interval. Return true if one was successfully expired */
     bool expireGRF(const GenRegInterval &limit);
@@ -84,6 +149,8 @@ namespace gbe
     GenContext &ctx;
     /*! Map virtual registers to offset in the (physical) register file */
     map<ir::Register, uint32_t> RA;
+    /*! Map offset to virtual registers. */
+    map<uint32_t, ir::Register> offsetReg;
     /*! Provides the position of each register in a vector */
     map<ir::Register, VectorLocation> vectorMap;
     /*! All vectors used in the selection */
@@ -97,29 +164,25 @@ namespace gbe
     /*! Intervals sorting based on ending point positions */
     vector<GenRegInterval*> ending;
     /*! registers that are spilled */
-    set<ir::Register> spilled;
+    SpilledRegs spilledRegs;
+    /*! register which could be spilled.*/
+    SpillCandidateSet spillCandidate;
     /* reserved registers for register spill/reload */
     uint32_t reservedReg;
     /*! Current vector to expire */
     uint32_t expiringID;
+    INLINE void insertNewReg(ir::Register reg, uint32_t grfOffset, bool isVector = false);
+    INLINE bool expireReg(ir::Register reg);
+    INLINE bool spillAtInterval(GenRegInterval interval, int size, uint32_t alignment);
+    INLINE uint32_t allocateReg(GenRegInterval interval, uint32_t size, uint32_t alignment);
+    INLINE bool spillReg(GenRegInterval interval, bool isAllocated = false);
+    INLINE bool spillReg(ir::Register reg, bool isAllocated = false);
+    INLINE bool vectorCanSpill(SelectionVector *vector);
+
     /*! Use custom allocator */
     GBE_CLASS(Opaque);
   };
 
-  // Note that byte vector registers use two bytes per byte (and can be
-  // interleaved)
-  static const size_t familyVectorSize[] = {2,2,2,4,8};
-  static const size_t familyScalarSize[] = {2,1,2,4,8};
-
-  /*! Interval as used in linear scan allocator. Basically, stores the first and
-   *  the last instruction where the register is alive
-   */
-  struct GenRegInterval {
-    INLINE GenRegInterval(ir::Register reg) :
-      reg(reg), minID(INT_MAX), maxID(-INT_MAX) {}
-    ir::Register reg;     //!< (virtual) register of the interval
-    int32_t minID, maxID; //!< Starting and ending points
-  };
 
   GenRegAllocator::Opaque::Opaque(GenContext &ctx) : ctx(ctx) {}
   GenRegAllocator::Opaque::~Opaque(void) {}
@@ -140,8 +203,7 @@ namespace gbe
   INLINE void GenRegAllocator::Opaque::allocatePayloadRegs(void) {
     using namespace ir;
     for(auto &it : this->ctx.curbeRegs)
-      if (it.first.value() < 0x8000)
-        allocatePayloadReg(it.first, it.second);
+      allocatePayloadReg(it.first, it.second);
 
     // Allocate all pushed registers (i.e. structure kernel arguments)
     const Function &fn = ctx.getFunction();
@@ -163,22 +225,19 @@ namespace gbe
   bool GenRegAllocator::Opaque::createGenReg(const GenRegInterval &interval) {
     using namespace ir;
     const ir::Register reg = interval.reg;
-    const uint32_t simdWidth = ctx.getSimdWidth();
     if (RA.contains(reg) == true)
       return true; // already allocated
     GBE_ASSERT(ctx.isScalarReg(reg) == false);
-    const bool isScalar = ctx.sel->isScalarOrBool(reg);
-    const RegisterData regData = ctx.sel->getRegisterData(reg);
-    const RegisterFamily family = regData.family;
-    const uint32_t typeSize = isScalar ? familyScalarSize[family] : familyVectorSize[family];
-    const uint32_t regSize = isScalar ? typeSize : simdWidth*typeSize;
-    uint32_t grfOffset;
-    while ((grfOffset = ctx.allocate(regSize, regSize)) == 0) {
-      const bool success = this->expireGRF(interval);
-      if (UNLIKELY(success == false)) return false;
+    uint32_t regSize;
+    ir::RegisterFamily family;
+    getRegAttrib(reg, regSize, &family);
+    uint32_t grfOffset = allocateReg(interval, regSize, regSize);
+    if (grfOffset == 0) {
+      /* this register is going to be spilled. */
+      GBE_ASSERT(!(reservedReg && family != ir::FAMILY_DWORD));
+      return false;
     }
-    GBE_ASSERTM(grfOffset != 0, "Unable to register allocate");
-    RA.insert(std::make_pair(reg, grfOffset));
+    insertNewReg(reg, grfOffset);
     return true;
   }
 
@@ -276,6 +335,7 @@ namespace gbe
   }
 
   bool GenRegAllocator::Opaque::expireGRF(const GenRegInterval &limit) {
+    bool ret = false;
     while (this->expiringID != ending.size()) {
       const GenRegInterval *toExpire = this->ending[this->expiringID];
       const ir::Register reg = toExpire->reg;
@@ -287,34 +347,26 @@ namespace gbe
       }
 
       //ignore register that already spilled
-      if(spilled.contains(reg)) {
+      if(spilledRegs.find(reg) != spilledRegs.end()) {
         this->expiringID++;
         continue;
       }
       // Ignore booleans that were allocated with flags
-      // if (ctx.getRegisterFamily(reg) == ir::FAMILY_BOOL && !grfBooleans.contains(reg)) {
-      if (ctx.sel->getRegisterFamily(reg) == ir::FAMILY_BOOL) {
+      if (ctx.sel->getRegisterFamily(reg) == ir::FAMILY_BOOL && !grfBooleans.contains(reg)) {
         this->expiringID++;
         continue;
       }
 
       if (toExpire->maxID >= limit.minID)
-        return false;
-      auto it = RA.find(reg);
-      GBE_ASSERT(it != RA.end());
-      // offset less than 32 means it is not managed by our reg allocator.
-      if (it->second < 32) {
-        this->expiringID++;
-        continue;
-      }
-      // Case 1 - it does not belong to a vector. Just remove it
-        ctx.deallocate(it->second);
-        this->expiringID++;
-        return true;
+        break;
+
+      if (expireReg(reg))
+        ret = true;
+      this->expiringID++;
     }
 
     // We were not able to expire anything
-    return false;
+    return ret;
   }
 
   void GenRegAllocator::Opaque::allocateFlags(Selection &selection) {
@@ -384,9 +436,10 @@ namespace gbe
             grfBooleans.insert(spill.reg);
             spill = interval;
           }
-          // We will a grf for the current register
-          else
+          // We will use a grf for the current register
+          else {
             grfBooleans.insert(reg);
+          }
         }
         else
           allocatedFlags.insert(std::make_pair(reg, freeFlags[--freeNum]));
@@ -481,42 +534,30 @@ namespace gbe
       if (RA.contains(reg))
         continue; // already allocated
 
+      if (ctx.sel->getRegisterFamily(reg) == ir::FAMILY_BOOL && !grfBooleans.contains(reg))
+        continue;
+
       // Case 1: the register belongs to a vector, allocate all the registers in
       // one piece
       auto it = vectorMap.find(reg);
       if (it != vectorMap.end()) {
         const SelectionVector *vector = it->second.first;
         // all the reg in the SelectionVector are spilled
-        if(spilled.contains(vector->reg[0].reg()))
+        if(spilledRegs.find(vector->reg[0].reg())
+           != spilledRegs.end())
           continue;
-        const uint32_t simdWidth = ctx.getSimdWidth();
-
-        const ir::RegisterData regData = ctx.sel->getRegisterData(reg);
-        const ir::RegisterFamily family = regData.family;
-        const uint32_t typeSize = familyVectorSize[family];
-        const uint32_t alignment = simdWidth*typeSize;
 
+        uint32_t alignment;
+        ir::RegisterFamily family;
+        getRegAttrib(reg, alignment, &family);
         const uint32_t size = vector->regNum * alignment;
-
-        uint32_t grfOffset;
-        while ((grfOffset = ctx.allocate(size, alignment)) == 0) {
-          const bool success = this->expireGRF(interval);
-          if (success == false) {
-            // if no spill support, just return false, else simply spill the register
-            if(reservedReg == 0) return false;
-            break;
-          }
-        }
+        const uint32_t grfOffset = allocateReg(interval, size, alignment);
         if(grfOffset == 0) {
-          // spill all the registers in the SelectionVector
-          // the tricky here is I need to use reservedReg+1 as scratch write payload.
-          // so, i need to write the first register to scratch memory first.
-          // the spillReg() will just append scratch write insn after the def. To spill
-          // the first register, need to call spillReg() last for the vector->reg[0]
+          GBE_ASSERT(!(reservedReg && family != ir::FAMILY_DWORD));
           GBE_ASSERT(vector->regNum < RESERVED_REG_NUM_FOR_SPILL);
           for(int i = vector->regNum-1; i >= 0; i--) {
-            spilled.insert(vector->reg[i].reg());
-            selection.spillReg(vector->reg[i].reg(), reservedReg);
+            if (!spillReg(vector->reg[i].reg()))
+              return false;
           }
           continue;
         }
@@ -524,20 +565,181 @@ namespace gbe
           const ir::Register reg = vector->reg[regID].reg();
           GBE_ASSERT(RA.contains(reg) == false
                      && ctx.sel->getRegisterData(reg).family == family);
-          RA.insert(std::make_pair(reg, grfOffset + alignment * regID));
+          insertNewReg(reg, grfOffset + alignment * regID, true);
           ctx.splitBlock(grfOffset, alignment * regID);  //splitBlock will not split if regID == 0
         }
       }
       // Case 2: This is a regular scalar register, allocate it alone
       else if (this->createGenReg(interval) == false) {
-        if(reservedReg == 0) return false;
-        spilled.insert(reg);
-        selection.spillReg(reg, reservedReg);
+        if (!spillReg(interval))
+          return false;
+      }
+    }
+    if (!spilledRegs.empty()) {
+      GBE_ASSERT(reservedReg != 0);
+      bool success = selection.spillRegs(spilledRegs, reservedReg);
+      if (!success) {
+        std::cerr << "Fail to spill registers." << std::endl;
+        return false;
       }
     }
     return true;
   }
 
+  INLINE bool GenRegAllocator::Opaque::expireReg(ir::Register reg)
+  {
+    auto it = RA.find(reg);
+    GBE_ASSERT(it != RA.end());
+    // offset less than 32 means it is not managed by our reg allocator.
+    if (it->second < 32)
+      return false;
+
+    ctx.deallocate(it->second);
+    if (reservedReg != 0
+        && (spillCandidate.find(intervals[reg]) != spillCandidate.end())) {
+        spillCandidate.erase(intervals[reg]);
+        /* offset --> reg map should keep updated. */
+        offsetReg.erase(it->second);
+    }
+
+    return true;
+  }
+
+  // insert a new register with allocated offset,
+  // put it to the RA map and the spill map if it could be spilled.
+  INLINE void GenRegAllocator::Opaque::insertNewReg(ir::Register reg, uint32_t grfOffset, bool isVector)
+  {
+     RA.insert(std::make_pair(reg, grfOffset));
+
+     if (reservedReg != 0) {
+
+       uint32_t regSize;
+       ir::RegisterFamily family;
+       getRegAttrib(reg, regSize, &family);
+
+       if (regSize == GEN_REG_SIZE && family == ir::FAMILY_DWORD /*&& !isVector*/) {
+         GBE_ASSERT(offsetReg.find(grfOffset) == offsetReg.end());
+         offsetReg.insert(std::make_pair(grfOffset, reg));
+         spillCandidate.insert(intervals[reg]);
+       }
+     }
+  }
+
+  INLINE bool GenRegAllocator::Opaque::spillReg(ir::Register reg,
+                                                bool isAllocated) {
+    return spillReg(intervals[reg], isAllocated);
+  }
+
+  INLINE bool GenRegAllocator::Opaque::spillReg(GenRegInterval interval,
+                                                bool isAllocated) {
+    if (reservedReg == 0)
+      return false;
+    SpillRegTag spillTag;
+    spillTag.isTmpReg = interval.maxID == interval.minID;
+    if (!spillTag.isTmpReg) {
+      // FIXME, we can optimize scratch allocation according to
+      // the interval information.
+      spillTag.addr = ctx.allocateScratchMem(typeSize(GEN_TYPE_D)
+                                             * ctx.getSimdWidth());
+    } else
+      spillTag.addr = -1;
+    if (isAllocated) {
+      // If this register is allocated, we need to expire it and erase it
+      // from the RA map.
+      bool success = expireReg(interval.reg);
+      GBE_ASSERT(success);
+      RA.erase(interval.reg);
+    }
+    spilledRegs.insert(std::make_pair(interval.reg, spillTag));
+    return true;
+  }
+
+  INLINE bool GenRegAllocator::Opaque::vectorCanSpill(SelectionVector *vector) {
+    for(uint32_t id = 0; id < vector->regNum; id++)
+      if (spillCandidate.find(intervals[(ir::Register)(vector->reg[id]).value.reg])
+          == spillCandidate.end())
+        return false;
+    return true;
+  }
+
+  INLINE bool GenRegAllocator::Opaque::spillAtInterval(GenRegInterval interval,
+                                                       int size,
+                                                       uint32_t alignment) {
+    if (reservedReg == 0)
+      return false;
+    auto it = spillCandidate.begin();
+    // If there is no spill candidate or current register is spillable and current register's
+    // endpoint is after all the spillCandidate register's endpoint we return false. The
+    // caller will spill current register.
+    if (it == spillCandidate.end()
+        || (it->getMaxID() <= interval.maxID && alignment == GEN_REG_SIZE))
+      return false;
+
+    ir::Register reg = it->getReg();
+    set<ir::Register> spillSet;
+    int32_t savedSize = size;
+    while(size > 0) {
+      auto vectorIt = vectorMap.find(reg);
+      bool isVector = vectorIt != vectorMap.end();
+      bool needRestart = false;
+      if (isVector
+          && (vectorCanSpill(vectorIt->second.first))) {
+        const SelectionVector *vector = vectorIt->second.first;
+        for (uint32_t id = 0; id < vector->regNum; id++) {
+          GBE_ASSERT(spilledRegs.find(vector->reg[id].reg())
+                     == spilledRegs.end());
+          spillSet.insert(vector->reg[id].reg());
+          reg = vector->reg[id].reg();
+          size -= GEN_REG_SIZE;
+        }
+      } else if (!isVector) {
+        spillSet.insert(reg);
+        size -= GEN_REG_SIZE;
+      } else
+        needRestart = true; // is a vector which could not be spilled.
+
+      if (size <= 0)
+        break;
+      if (!needRestart) {
+        uint32_t offset = RA.find(reg)->second;
+        auto nextRegIt = offsetReg.find(offset + GEN_REG_SIZE);
+        if (nextRegIt != offsetReg.end())
+          reg = nextRegIt->second;
+        else
+          needRestart = true;
+      }
+
+      if (needRestart) {
+        // next register is not in spill candidate.
+        // let's move to next candidate and start over.
+        it++;
+        if (it == spillCandidate.end())
+          return false;
+        reg = it->getReg();
+        size = savedSize;
+        spillSet.clear();
+      }
+    }
+
+    for(auto spillreg : spillSet)
+      spillReg(spillreg, true);
+    return true;
+  }
+
+  INLINE uint32_t GenRegAllocator::Opaque::allocateReg(GenRegInterval interval,
+                                                       uint32_t size,
+                                                       uint32_t alignment) {
+    uint32_t grfOffset;
+    while ((grfOffset = ctx.allocate(size, alignment)) == 0) {
+      const bool success = this->expireGRF(interval);
+      if (success == false) {
+        if (spillAtInterval(interval, size, alignment) == false)
+          return 0;
+      }
+    }
+    return grfOffset;
+  }
+
   INLINE bool GenRegAllocator::Opaque::allocate(Selection &selection) {
     using namespace ir;
     if (ctx.getSimdWidth() == 8) {
@@ -569,6 +771,7 @@ namespace gbe
     int32_t insnID = 0;
     for (auto &block : *selection.blockList) {
       int32_t lastID = insnID;
+      int32_t firstID = insnID;
       // Update the intervals of each used register. Note that we do not
       // register allocate R0, so we skip all sub-registers in r0
       for (auto &insn : block.insnList) {
@@ -619,16 +822,29 @@ namespace gbe
         insnID++;
       }
 
+      // All registers alive at the begining of the block must update their intervals.
+      const ir::BasicBlock *bb = block.bb;
+      for (auto reg : ctx.getLiveIn(bb))
+          this->intervals[reg].minID = std::min(this->intervals[reg].minID, firstID);
+
+      for (auto reg : ctx.getExtraLiveIn(bb))
+          this->intervals[reg].minID = std::min(this->intervals[reg].minID, firstID);
       // All registers alive at the end of the block must have their intervals
       // updated as well
-      const ir::BasicBlock *bb = block.bb;
-      const ir::Liveness::LiveOut &liveOut = ctx.getLiveOut(bb);
-      for (auto reg : liveOut) {
-        this->intervals[reg].minID = std::min(this->intervals[reg].minID, lastID);
+      for (auto reg : ctx.getLiveOut(bb))
+        this->intervals[reg].maxID = std::max(this->intervals[reg].maxID, lastID);
+
+      for (auto reg : ctx.getExtraLiveOut(bb))
         this->intervals[reg].maxID = std::max(this->intervals[reg].maxID, lastID);
-      }
     }
 
+    this->intervals[ocl::emask].minID = 0;
+    this->intervals[ocl::emask].maxID = INT_MAX;
+    this->intervals[ocl::notemask].minID = 0;
+    this->intervals[ocl::notemask].maxID = INT_MAX;
+    this->intervals[ocl::retVal].minID = INT_MAX;
+    this->intervals[ocl::retVal].maxID = -INT_MAX;
+
     // Sort both intervals in starting point and ending point increasing orders
     const uint32_t regNum = ctx.sel->getRegNum();
     this->starting.resize(regNum);
@@ -657,19 +873,40 @@ namespace gbe
   }
 
   INLINE void GenRegAllocator::Opaque::outputAllocation(void) {
-    std::cout << "## register allocation ##" << std::endl;
+    using namespace std;
+    cout << "## register allocation ##" << endl;
     for(auto &i : RA) {
-        int vReg = (int)i.first;
-        int offst = (int)i.second / sizeof(float);
-        int reg = offst / 8;
-        int subreg = offst % 8;
-        std::cout << "%" << vReg << " g" << reg << "." << subreg << "D" << std::endl;
+        ir::Register vReg = (ir::Register)i.first;
+        ir::RegisterFamily family;
+        uint32_t regSize;
+        getRegAttrib(vReg, regSize, &family);
+        int offst = (int)i.second;// / sizeof(float);
+        int reg = offst / 32;
+        int subreg = (offst % 32) / regSize;
+        cout << "%" << setiosflags(ios::left) << setw(8) << vReg
+             << "g" << setiosflags(ios::left) << setw(3) << reg << "."
+             << setiosflags(ios::left) << setw(3) << subreg << ir::getFamilyName(family)
+             << "  " << setw(-3) << regSize  << "B\t"
+             << "[  " << setw(8) << this->intervals[(uint)vReg].minID
+             << " -> " << setw(8) << this->intervals[(uint)vReg].maxID
+             << "]" << endl;
     }
-    std::set<ir::Register>::iterator is;
-    std::cout << "## spilled registers:" << std::endl;
-    for(is = spilled.begin(); is != spilled.end(); is++)
-      std::cout << (int)*is << std::endl;
-    std::cout << std::endl;
+    if (!spilledRegs.empty())
+      cout << "## spilled registers: " << spilledRegs.size() << endl;
+    for(auto it = spilledRegs.begin(); it != spilledRegs.end(); it++) {
+      ir::Register vReg = it->first;
+      ir::RegisterFamily family;
+      uint32_t regSize;
+      getRegAttrib(vReg, regSize, &family);
+      cout << "%" << setiosflags(ios::left) << setw(8) << vReg
+           << "@" << setw(8) << it->second.addr
+           << "  " << ir::getFamilyName(family)
+           <<  "  " << setw(-3) << regSize << "B\t"
+           << "[  " << setw(8) << this->intervals[(uint)vReg].minID
+           << " -> " << setw(8) << this->intervals[(uint)vReg].maxID
+           << "]" << endl;
+    }
+    cout << endl;
   }
 
   INLINE GenRegister setGenReg(const GenRegister &src, uint32_t grfOffset) {
