@@ -243,19 +243,53 @@ namespace ir {
       INLINE bool wellFormed(const Function &fn, std::string &whyNot) const;
     };
 
+    class ALIGNED_INSTRUCTION BitCastInstruction :
+      public BasePolicy,
+      public TupleSrcPolicy<BitCastInstruction>,
+      public TupleDstPolicy<BitCastInstruction>
+    {
+    public:
+      BitCastInstruction(Type dstType,
+                         Type srcType,
+                         Tuple dst,
+                         Tuple src,
+                         uint8_t dstNum,
+                         uint8_t srcNum)
+      {
+        this->opcode = OP_BITCAST;
+        this->dst = dst;
+        this->src = src;
+        this->dstFamily = getFamily(dstType);
+        this->srcFamily = getFamily(srcType);
+        GBE_ASSERT(srcNum <= 16 && dstNum <= 16);
+        this->dstNum = dstNum;
+        this->srcNum = srcNum;
+      }
+      INLINE Type getSrcType(void) const { return getType((RegisterFamily)srcFamily); }
+      INLINE Type getDstType(void) const { return getType((RegisterFamily)dstFamily); }
+      INLINE bool wellFormed(const Function &fn, std::string &whyNot) const;
+      INLINE void out(std::ostream &out, const Function &fn) const;
+      uint8_t dstFamily:4; //!< family to cast to
+      uint8_t srcFamily:4; //!< family to cast from
+      Tuple dst;
+      Tuple src;
+      uint8_t dstNum;     //!<Dst Number
+      uint8_t srcNum;     //!<Src Number
+    };
+
     class ALIGNED_INSTRUCTION ConvertInstruction :
       public BasePolicy,
       public NDstPolicy<ConvertInstruction, 1>,
       public NSrcPolicy<ConvertInstruction, 1>
     {
     public:
-      ConvertInstruction(Type dstType,
+      ConvertInstruction(Opcode opcode,
+                         Type dstType,
                          Type srcType,
                          Register dst,
-                         Register src,
-                         bool saturated=false)
+                         Register src)
       {
-        this->opcode = saturated ? OP_SAT_CVT : OP_CVT;
+        this->opcode = opcode;
         this->dst[0] = dst;
         this->src[0] = src;
         this->dstType = dstType;
@@ -457,37 +491,48 @@ namespace ir {
       public TupleDstPolicy<SampleInstruction>
     {
     public:
-      SampleInstruction(Tuple dstTuple, Tuple srcTuple, Type dstType, Type srcType) {
+      SampleInstruction(uint8_t imageIdx, Tuple dstTuple, Tuple srcTuple, bool dstIsFloat, bool srcIsFloat, uint8_t sampler, uint8_t samplerOffset, bool is3D) {
         this->opcode = OP_SAMPLE;
         this->dst = dstTuple;
         this->src = srcTuple;
-        this->dstType = dstType;
-        this->srcType = srcType;
+        this->dstIsFloat = dstIsFloat;
+        this->srcIsFloat = srcIsFloat;
+        this->samplerIdx = sampler;
+        this->imageIdx = imageIdx;
+        this->samplerOffset = samplerOffset;
+        this->is3DRead = is3D;
       }
       INLINE bool wellFormed(const Function &fn, std::string &why) const;
       INLINE void out(std::ostream &out, const Function &fn) const {
         this->outOpcode(out);
         out << "." << this->getDstType()
             << "." << this->getSrcType()
-            << " surface id %" << this->getSrc(fn, 0)
-            << " sampler %" << this->getSrc(fn, 1)
-            << " coord u %" << this->getSrc(fn, 2)
-            << " coord v %" << this->getSrc(fn, 3)
-            << " coord w %" << this->getSrc(fn, 4)
+            << " surface id " << (int)this->getImageIndex()
+            << " coord u %" << this->getSrc(fn, 0)
+            << " coord v %" << this->getSrc(fn, 1)
+            << " coord w %" << this->getSrc(fn, 2)
             << " %" << this->getDst(fn, 0)
             << " %" << this->getDst(fn, 1)
             << " %" << this->getDst(fn, 2)
-            << " %" << this->getDst(fn, 3);
+            << " %" << this->getDst(fn, 3)
+            << " sampler idx " << (int)this->getSamplerIndex();
       }
       Tuple src;
       Tuple dst;
-      Type srcType;
-      Type dstType;
 
-      INLINE Type getSrcType(void) const { return this->srcType; }
-      INLINE Type getDstType(void) const { return this->dstType; }
-
-      static const uint32_t srcNum = 6;
+      INLINE const uint8_t getImageIndex(void) const { return this->imageIdx; }
+      INLINE Type getSrcType(void) const { return this->srcIsFloat ? TYPE_FLOAT : TYPE_S32; }
+      INLINE Type getDstType(void) const { return this->dstIsFloat ? TYPE_FLOAT : TYPE_U32; }
+      INLINE const uint8_t getSamplerIndex(void) const { return this->samplerIdx; }
+      INLINE const uint8_t getSamplerOffset(void) const { return this->samplerOffset; }
+      INLINE const bool is3D(void) const { return !!this->is3DRead; }
+      uint8_t srcIsFloat:1;
+      uint8_t dstIsFloat:1;
+      uint8_t samplerIdx:4;
+      uint8_t samplerOffset:1;
+      uint8_t is3DRead:1;
+      uint8_t imageIdx;
+      static const uint32_t srcNum = 3;
       static const uint32_t dstNum = 4;
     };
 
@@ -498,34 +543,41 @@ namespace ir {
     {
     public:
 
-      INLINE TypedWriteInstruction(Tuple srcTuple, Type srcType, Type coordType) {
+      INLINE TypedWriteInstruction(uint8_t imageIdx, Tuple srcTuple, Type srcType, Type coordType, bool is3D) {
         this->opcode = OP_TYPED_WRITE;
         this->src = srcTuple;
         this->coordType = coordType;
         this->srcType = srcType;
+        this->imageIdx = imageIdx;
+        this->is3DWrite = is3D;
       }
       INLINE bool wellFormed(const Function &fn, std::string &why) const;
       INLINE void out(std::ostream &out, const Function &fn) const {
         this->outOpcode(out);
         out << "." << this->getSrcType()
-            << " surface id %" << this->getSrc(fn, 0)
-            << " coord u %" << this->getSrc(fn, 1)
-            << " coord v %" << this->getSrc(fn, 2)
-            << " coord w %" << this->getSrc(fn, 3)
+            << " surface id " << (int)this->getImageIndex()
+            << " coord u %" << this->getSrc(fn, 0)
+            << " coord v %" << this->getSrc(fn, 1)
+            << " coord w %" << this->getSrc(fn, 2)
+            << " %" << this->getSrc(fn, 3)
             << " %" << this->getSrc(fn, 4)
             << " %" << this->getSrc(fn, 5)
-            << " %" << this->getSrc(fn, 6)
-            << " %" << this->getSrc(fn, 7);
+            << " %" << this->getSrc(fn, 6);
       }
 
       Tuple src;
-      Type srcType;
-      Type coordType;
+      uint8_t srcType;
+      uint8_t coordType;
+      uint8_t imageIdx;
+      uint8_t is3DWrite;
 
-      INLINE Type getSrcType(void) const { return this->srcType; }
-      INLINE Type getCoordType(void) const { return this->coordType; }
+      INLINE const bool is3D(void) const { return !!this->is3DWrite; }
+
+      INLINE const uint8_t getImageIndex(void) const { return this->imageIdx; }
+      INLINE Type getSrcType(void) const { return (Type)this->srcType; }
+      INLINE Type getCoordType(void) const { return (Type)this->coordType; }
       // bti, u, v, w, 4 data elements
-      static const uint32_t srcNum = 8;
+      static const uint32_t srcNum = 7;
       Register dst[0];               //!< No dest register
     };
 
@@ -536,41 +588,48 @@ namespace ir {
     {
     public:
       GetSamplerInfoInstruction( Register dst,
-                                 Register src)
+                                 Register samplerInfo,
+                                 uint8_t samplerIdx)
       {
         this->opcode = OP_GET_SAMPLER_INFO;
         this->dst[0] = dst;
-        this->src[0] = src;
+        this->src[0] = samplerInfo;
+        this->samplerIdx = samplerIdx;
       }
 
       INLINE bool wellFormed(const Function &fn, std::string &why) const;
       INLINE void out(std::ostream &out, const Function &fn) const {
         this->outOpcode(out);
-        out << " sampler id %" << this->getSrc(fn, 0)
-            << " %" << this->getDst(fn, 0);
+        out  << " %" << this->getDst(fn, 0)
+             << " %" << this->getSrc(fn, 0)
+             << " sampler idx " << (int)this->samplerIdx;
+      }
+      INLINE const uint8_t getSamplerIndex() const {
+        return this->samplerIdx;
       }
 
-      Register src[1];                  //!< Surface to get info
+      Register src[1];                  //!< sampler to get info
       Register dst[1];                  //!< return value
+      uint8_t samplerIdx;               //!< sampler slot index.
       static const uint32_t dstNum = 1;
     };
 
     class ALIGNED_INSTRUCTION GetImageInfoInstruction :
       public BasePolicy,
-      public NSrcPolicy<GetImageInfoInstruction, 2>,
+      public NSrcPolicy<GetImageInfoInstruction, 1>,
       public NDstPolicy<GetImageInfoInstruction, 1>
     {
     public:
       GetImageInfoInstruction( int type,
                                Register dst,
-                               Register src,
+                               uint8_t imageIdx,
                                Register infoReg)
       {
         this->opcode = OP_GET_IMAGE_INFO;
         this->infoType = type;
         this->dst[0] = dst;
-        this->src[0] = src;
-        this->src[1] = infoReg;
+        this->src[0] = infoReg;
+        this->imageIdx = imageIdx;
       }
 
       INLINE uint32_t getInfoType(void) const { return infoType; }
@@ -578,13 +637,17 @@ namespace ir {
       INLINE void out(std::ostream &out, const Function &fn) const {
         this->outOpcode(out);
         out << "." << this->getInfoType()
-            << " surface id %" << this->getSrc(fn, 0)
-            << " %" << this->getDst(fn, 0);
+            << " %" << this->getDst(fn, 0)
+            << " surface id " << (int)this->getImageIndex()
+            << " info reg %" << this->getSrc(fn, 0);
       }
 
+      INLINE const uint8_t getImageIndex(void) const { return imageIdx; }
+
       uint8_t infoType;                 //!< Type of the requested information.
-      Register src[2];                  //!< Surface to get info
-      Register dst[1];                        //!< dest register to put the information.
+      uint8_t imageIdx;                //!< surface index.
+      Register src[1];                  //!< surface info register.
+      Register dst[1];                  //!< dest register to put the information.
       static const uint32_t dstNum = 1;
     };
 
@@ -750,6 +813,8 @@ namespace ir {
         default:
           CHECK_TYPE(this->type, allButBool);
           break;
+        case OP_MOV:
+          break;
         case OP_POW:
         case OP_COS:
         case OP_SIN:
@@ -809,6 +874,35 @@ namespace ir {
       return true;
     }
 
+    // The bit sizes of src and the dst must be identical, and don't support bool now, bool need double check.
+    INLINE bool BitCastInstruction::wellFormed(const Function &fn, std::string &whyNot) const
+    {
+      for (uint32_t dstID = 0; dstID < dstNum; ++dstID) {
+        if (UNLIKELY(checkSpecialRegForWrite(getDst(fn, dstID), fn, whyNot) == false))
+          return false;
+        if (UNLIKELY(checkRegisterData((RegisterFamily)dstFamily, getDst(fn, dstID), fn, whyNot) == false))
+          return false;
+      }
+      for (uint32_t srcID = 0; srcID < srcNum; ++srcID) {
+        if (UNLIKELY(checkRegisterData((RegisterFamily)srcFamily, getSrc(fn, srcID), fn, whyNot) == false))
+          return false;
+      }
+
+      CHECK_TYPE(getType((RegisterFamily)dstFamily), allButBool);
+      CHECK_TYPE(getType((RegisterFamily)srcFamily), allButBool);
+
+      uint32_t dstBytes = 0, srcBtyes = 0;
+      dstBytes = dstNum * getFamilySize((RegisterFamily)dstFamily);
+      srcBtyes = srcNum * getFamilySize((RegisterFamily)srcFamily);
+
+      if(dstBytes != srcBtyes){
+        whyNot = " The bit sizes of src and the dst is not identical.";
+        return false;
+      }
+
+      return true;
+    }
+
     // We can convert anything to anything, but types and families must match
     INLINE bool ConvertInstruction::wellFormed(const Function &fn, std::string &whyNot) const
     {
@@ -1020,6 +1114,22 @@ namespace ir {
         out << " %" << this->getSrc(fn, i);
     }
 
+
+    INLINE void BitCastInstruction::out(std::ostream &out, const Function &fn) const {
+      this->outOpcode(out);
+      out << "." << this->getDstType()
+          << "." << this->getSrcType();
+      out << " {";
+      for (uint32_t i = 0; i < dstNum; ++i)
+        out << "%" << this->getDst(fn, i) << (i != (dstNum-1u) ? " " : "");
+      out << "}";
+      out << " {";
+      for (uint32_t i = 0; i < srcNum; ++i)
+        out << "%" << this->getSrc(fn, i) << (i != (srcNum-1u) ? " " : "");
+      out << "}";
+    }
+
+
     INLINE void ConvertInstruction::out(std::ostream &out, const Function &fn) const {
       this->outOpcode(out);
       out << "." << this->getDstType()
@@ -1142,6 +1252,10 @@ START_INTROSPECTION(CompareInstruction)
 #include "ir/instruction.hxx"
 END_INTROSPECTION(CompareInstruction)
 
+START_INTROSPECTION(BitCastInstruction)
+#include "ir/instruction.hxx"
+END_INTROSPECTION(BitCastInstruction)
+
 START_INTROSPECTION(ConvertInstruction)
 #include "ir/instruction.hxx"
 END_INTROSPECTION(ConvertInstruction)
@@ -1328,6 +1442,15 @@ END_FUNCTION(Instruction, Register)
     fn.deleteInstruction(this);
   }
 
+  void Instruction::insert(Instruction *prev, Instruction ** new_ins) {
+    Function &fn = prev->getFunction();
+    Instruction *insn = fn.newInstruction(*this);
+    insn->parent = prev->parent;
+    append(insn, prev);
+    if (new_ins)
+      *new_ins = insn;
+  }
+
   bool Instruction::hasSideEffect(void) const {
     return opcode == OP_STORE ||
            opcode == OP_TYPED_WRITE ||
@@ -1346,6 +1469,8 @@ DECL_MEM_FN(BinaryInstruction, bool, commutes(void), commutes())
 DECL_MEM_FN(SelectInstruction, Type, getType(void), getType())
 DECL_MEM_FN(TernaryInstruction, Type, getType(void), getType())
 DECL_MEM_FN(CompareInstruction, Type, getType(void), getType())
+DECL_MEM_FN(BitCastInstruction, Type, getSrcType(void), getSrcType())
+DECL_MEM_FN(BitCastInstruction, Type, getDstType(void), getDstType())
 DECL_MEM_FN(ConvertInstruction, Type, getSrcType(void), getSrcType())
 DECL_MEM_FN(ConvertInstruction, Type, getDstType(void), getDstType())
 DECL_MEM_FN(AtomicInstruction, AddressSpace, getAddressSpace(void), getAddressSpace())
@@ -1365,9 +1490,17 @@ DECL_MEM_FN(BranchInstruction, LabelIndex, getLabelIndex(void), getLabelIndex())
 DECL_MEM_FN(SyncInstruction, uint32_t, getParameters(void), getParameters())
 DECL_MEM_FN(SampleInstruction, Type, getSrcType(void), getSrcType())
 DECL_MEM_FN(SampleInstruction, Type, getDstType(void), getDstType())
+DECL_MEM_FN(SampleInstruction, const uint8_t, getSamplerIndex(void), getSamplerIndex())
+DECL_MEM_FN(SampleInstruction, const bool, is3D(void), is3D())
+DECL_MEM_FN(SampleInstruction, const uint8_t, getSamplerOffset(void), getSamplerOffset())
+DECL_MEM_FN(SampleInstruction, const uint8_t, getImageIndex(void), getImageIndex())
 DECL_MEM_FN(TypedWriteInstruction, Type, getSrcType(void), getSrcType())
 DECL_MEM_FN(TypedWriteInstruction, Type, getCoordType(void), getCoordType())
+DECL_MEM_FN(TypedWriteInstruction, const uint8_t, getImageIndex(void), getImageIndex())
+DECL_MEM_FN(TypedWriteInstruction, const bool, is3D(void), is3D())
 DECL_MEM_FN(GetImageInfoInstruction, uint32_t, getInfoType(void), getInfoType())
+DECL_MEM_FN(GetImageInfoInstruction, const uint8_t, getImageIndex(void), getImageIndex())
+DECL_MEM_FN(GetSamplerInfoInstruction, const uint8_t, getSamplerIndex(void), getSamplerIndex())
 
 #undef DECL_MEM_FN
 
@@ -1449,6 +1582,9 @@ DECL_MEM_FN(GetImageInfoInstruction, uint32_t, getInfoType(void), getInfoType())
     return internal::TernaryInstruction(OP_I64MADSAT, type, dst, src).convert();
   }
 
+  Instruction MAD(Type type, Register dst, Tuple src) {
+    return internal::TernaryInstruction(OP_MAD, type, dst, src).convert();
+  }
   // All compare functions
 #define DECL_EMIT_FUNCTION(NAME) \
   Instruction NAME(Type type, Register dst,  Register src0, Register src1) { \
@@ -1462,17 +1598,33 @@ DECL_MEM_FN(GetImageInfoInstruction, uint32_t, getInfoType(void), getInfoType())
   DECL_EMIT_FUNCTION(LT)
   DECL_EMIT_FUNCTION(GE)
   DECL_EMIT_FUNCTION(GT)
+  DECL_EMIT_FUNCTION(ORD)
 
 #undef DECL_EMIT_FUNCTION
 
+  // BITCAST
+  Instruction BITCAST(Type dstType, Type srcType, Tuple dst, Tuple src, uint8_t dstNum, uint8_t srcNum) {
+    return internal::BitCastInstruction(dstType, srcType, dst, src, dstNum, srcNum).convert();
+  }
+
   // CVT
   Instruction CVT(Type dstType, Type srcType, Register dst, Register src) {
-    return internal::ConvertInstruction(dstType, srcType, dst, src).convert();
+    return internal::ConvertInstruction(OP_CVT, dstType, srcType, dst, src).convert();
   }
 
   // saturated convert
   Instruction SAT_CVT(Type dstType, Type srcType, Register dst, Register src) {
-    return internal::ConvertInstruction(dstType, srcType, dst, src, true).convert();
+    return internal::ConvertInstruction(OP_SAT_CVT, dstType, srcType, dst, src).convert();
+  }
+
+  // CVT
+  Instruction F16TO32(Type dstType, Type srcType, Register dst, Register src) {
+    return internal::ConvertInstruction(OP_F16TO32, dstType, srcType, dst, src).convert();
+  }
+
+  // saturated convert
+  Instruction F32TO16(Type dstType, Type srcType, Register dst, Register src) {
+    return internal::ConvertInstruction(OP_F32TO16, dstType, srcType, dst, src).convert();
   }
 
   // For all unary functions with given opcode
@@ -1526,20 +1678,20 @@ DECL_MEM_FN(GetImageInfoInstruction, uint32_t, getInfoType(void), getInfoType())
   }
 
   // SAMPLE
-  Instruction SAMPLE(Tuple dst, Tuple src, Type dstType, Type srcType) {
-    return internal::SampleInstruction(dst, src, dstType, srcType).convert();
+  Instruction SAMPLE(uint8_t imageIndex, Tuple dst, Tuple src, bool dstIsFloat, bool srcIsFloat, uint8_t sampler, uint8_t samplerOffset, bool is3D) {
+    return internal::SampleInstruction(imageIndex, dst, src, dstIsFloat, srcIsFloat, sampler, samplerOffset, is3D).convert();
   }
 
-  Instruction TYPED_WRITE(Tuple src, Type srcType, Type coordType) {
-    return internal::TypedWriteInstruction(src, srcType, coordType).convert();
+  Instruction TYPED_WRITE(uint8_t imageIndex, Tuple src, Type srcType, Type coordType, bool is3D) {
+    return internal::TypedWriteInstruction(imageIndex, src, srcType, coordType, is3D).convert();
   }
 
-  Instruction GET_IMAGE_INFO(int infoType, Register dst, Register src, Register infoReg) {
-    return internal::GetImageInfoInstruction(infoType, dst, src, infoReg).convert();
+  Instruction GET_IMAGE_INFO(int infoType, Register dst, uint8_t imageIndex, Register infoReg) {
+    return internal::GetImageInfoInstruction(infoType, dst, imageIndex, infoReg).convert();
   }
 
-  Instruction GET_SAMPLER_INFO(Register dst, Register src) {
-    return internal::GetSamplerInfoInstruction(dst, src).convert();
+  Instruction GET_SAMPLER_INFO(Register dst, Register samplerInfo, uint8_t samplerIdx) {
+    return internal::GetSamplerInfoInstruction(dst, samplerInfo, samplerIdx).convert();
   }
 
   std::ostream &operator<< (std::ostream &out, const Instruction &insn) {
