@@ -84,8 +84,12 @@ namespace gbe
   /*! Remove the GEP instructions */
   llvm::BasicBlockPass *createRemoveGEPPass(const ir::Unit &unit);
 
+  /*! Scalarize all vector op instructions */
   llvm::FunctionPass* createScalarizePass();
 
+  /*! Convert the Intrinsic call to gen function */
+  llvm::BasicBlockPass *createIntrinsicLoweringPass();
+
 } /* namespace gbe */
 
 #endif /* __GBE_LLVM_GEN_BACKEND_HPP__ */
