@@ -30,8 +30,8 @@ uint32_t cl_to_clk(cl_bool normalized_coords,
                    cl_addressing_mode address,
                    cl_filter_mode filter)
 {
-  int clk_address;
-  int clk_filter;
+  int clk_address = CLK_ADDRESS_NONE;
+  int clk_filter = CLK_FILTER_NEAREST;
   switch (address) {
   case CL_ADDRESS_NONE: clk_address = CLK_ADDRESS_NONE; break;
   case CL_ADDRESS_CLAMP: clk_address = CLK_ADDRESS_CLAMP; break;
@@ -67,7 +67,7 @@ int cl_set_sampler_arg_slot(cl_kernel k, int index, cl_sampler sampler)
      }
     }
   }
-  assert(0);
+  return -1;
 }
 
 LOCAL cl_sampler
@@ -125,8 +125,8 @@ cl_sampler_delete(cl_sampler sampler)
       sampler->prev->next = sampler->next;
     if (sampler->next)
       sampler->next->prev = sampler->prev;
-    if (sampler->prev == NULL && sampler->next == NULL)
-      sampler->ctx->samplers = NULL;
+    if (sampler->ctx->samplers == sampler)
+      sampler->ctx->samplers = sampler->next;
   pthread_mutex_unlock(&sampler->ctx->sampler_lock);
   cl_context_delete(sampler->ctx);
 
